########################
### Date: 2013-10-12 ###
### Author: Wenbo Wu ###
########################

###########################################################################
### Outline:										###
### This file contains following components: 					###
### Section 1: Define some basic tool functions					###
### Section 2: Dimension reduction methods					###
### Section 3: Stable estimation procedure for structural dimension	###
### Section 4: Stable sparse estimation of central subspace			###	
### Section 5: Stable nonsparse estimation of central subspace		###
### Section 6: Enssemble method for sliced inverse methods			###
### Section 7: Examples (including analysis on Hitters' dataset)		###
###########################################################################

####################################################
######  Section 1: Useful functions   ##############
####################################################
### compute power of positive definite matrix ###
matpower = function(x,p){
	return(eigen(x)$vectors%*%diag(round((eigen(x)$values),8)^(p))%*%solve(eigen(x)$vectors))
}
center = function(x){
	return(sweep(x,2,colMeans(x)))
}
standardize = function(x){
	return(center(x)%*%matpower(cov(x),-0.5))
}
standardize_col = function(A){
	for (i in 1:ncol(A)){
		if (sqrt(drop((t(A[,i])%*%A[,i])))!=0) A[,i]= A[,i]/sqrt(drop((t(A[,i])%*%A[,i])))
	}
	return(A)
}
### compute vector correlation between matrix beta and beta_hat ###
vc = function(beta, beta_hat){
	beta = standardize_col(beta)
	beta_hat = standardize_col(beta_hat)
	return(sqrt(abs(det(t(beta)%*%beta_hat%*%t(beta_hat)%*%beta))))
}
switch_col = function(A, i,j){
	temp = A[,i]
	A[,i]=A[,j]
	A[,j]=temp
	return(A)
}

sign_beta = function(a,b){
	for (i in 1:ncol(b)){
		if (sum(abs(a[,i]-b[,i]))>sum(abs(-a[,i]-b[,i]))) b[,i]=-b[,i]
	}
	return(b)
}

order_beta = function(a,b){
	p = nrow(b)
	d = ncol(b)
	for (i in 1:d){
		v = numeric(d)
		for (j in i:d){
			v[j]=vc(matrix(a[,i],p,1),matrix(b[,j],p,1))
		}
		b = switch_col(b,i,min(which(v==max(v))))
	}
	return(b)
}
vec = function(inMat){
	outMat = matrix(0,length(inMat),1)
	for(i in 1:ncol(inMat)){
		begin = 1 + (i - 1) * nrow(inMat)
		end = begin + nrow(inMat) - 1
		outMat[begin:end]=inMat[,i]
	}
	return(outMat)
}

get_trace = function(A){
	return(sum(diag(A)))
}  

get_mode = function(x){
	n = length(unique(x))
	information = matrix(0,n,2)
	information[,1]=unique(x)
	for (i in 1:n){
		information[i,2]= length(x[x==information[i,1]])
	}
	mode = max(information[information[,2]==max(information[,2]),1])
	return(mode)
}

####################################################################
####### Section 2: dimension reduction methods   ###################
####################################################################

### Sliced Inverse Regression (S.I.R.) ####
### obtaining S.I.R. kernel matrix by evenly split the response range ###
get_sir_matrix_even_range = function(x,y,H){
	z = standardize(x)
	n = length(y)
	p = ncol(x)
	sliced_y = numeric(n)
	slice_prop = numeric(H)
	divpt = c(min(y)-0.1,min(y)+((range(y)[2]-range(y)[1])/H)*c(1:H))
	sir_matrix = matrix(0,p,p)
	for(i in 1:H){
		sliced_y[(y<=divpt[i+1])&(y>divpt[i])]=i
		slice_prop[i] = sum(sliced_y[sliced_y==i])/(n*i)
		slice_mean = matrix(colMeans(matrix(z[sliced_y==i,],length(z[sliced_y==i,])/p,p)))
		if (slice_prop[i]!=0)
		sir_matrix = sir_matrix + slice_prop[i]*slice_mean%*%t(slice_mean)
	}
	return(sir_matrix)
}
### obtaining S.I.R. estimate for a given dimension ###
sir_even_range = function(x,y,H,d){
	p = ncol(x)
	M = get_sir_matrix_even_range(x,y,H)
	beta_hat=matpower(cov(x),-0.5)%*%(eigen(M)$vectors)[,1:d]
	beta_hat = standardize_col(matrix(beta_hat,p,d))
	return(beta_hat)
}
### estimating dimension of central subspace based on S.I.R. kernel matrix ###
sir_d_even_range = function(x,y,H,alpha){
	M = get_sir_matrix_even_range(x,y,H)
	lambda = eigen(M)$values
	n = length(unique(y))
	p = ncol(x)
	test = matrix(0,p,4)
	k = prob = 0
	while((k<p)&&(prob<alpha)){
		lambda_bar = mean(lambda[(k+1):p])
		test[(k+1),1] = n*(p-k)*lambda_bar
		test[(k+1),2] = (p-k)*(H-k-1)
		test[(k+1),3] = 1-pchisq(test[(k+1),1],test[(k+1),2])
		test[(k+1),4] = k
		prob = test[(k+1),3]
		k = k+1
	}
	return(test[k,4])
}
### Principal Hessian Directions (P.H.D.) ####
### obtaining P.H.D. using residuals ###
get_phd_matrix_r = function(x,y){
	r = residuals(lm(y~x))
	xbar=colMeans(x)
	n = nrow(x)
	p = ncol(x)
	sigma_rxx = matrix(0,p,p)
	for(i in 1:n){
		sigma_rxx = sigma_rxx +r[i]*t(t(x[i,]-xbar))%*%t(x[i,]-xbar)
	}
	m_phd = solve(cov(x))%*%(sigma_rxx/n)

	return(m_phd)
}
get_phd_matrix = function(x,y){
	r = residuals(lm(y~x))
	xbar=colMeans(x)
	n = nrow(x)
	p = ncol(x)
	sigma_rxx = matrix(0,p,p)
	for(i in 1:n){
		sigma_rxx = sigma_rxx +r[i]*t(t(x[i,]-xbar))%*%t(x[i,]-xbar)
	}
	m_phd = solve(cov(x))%*%(sigma_rxx/n)

	return(m_phd%*%t(m_phd))
}
### obtaining P.H.D. estimate for a given dimension ###
phd_r = function(x,y,d){
	m_phd = get_phd_matrix_r(x,y)
	result = matpower(cov(x),-0.5)%*%eigen(m_phd)$vectors[,1:min(d,p)]
	return(standardize_col(matrix(result,p,d)))
}
### estimating dimension of central subspace based on P.H.D. kernel matrix ###
phd_d_r = function(x,y,alpha){
	m_phd = get_phd_matrix_r(x,y)
	lambda = eigen(m_phd)$values
	r = residuals(lm(y~x))
	n = nrow(x)
	p = ncol(x)
	msr = sum(r^2)/(n-p)
	test = matrix(0,p,4)
	K = prob = 0
	while((K<p)&&(prob<alpha)){
		test[(K+1),1] = n*sum(lambda[(K+1):p]^2)/(2*msr)
		test[(K+1),2] = (p-K+1)*(p-K)/2
		test[(K+1),3] = 1-pchisq(test[(K+1),1],test[(K+1),2])
		test[(K+1),4] = K
		prob = test[(K+1),3]
		K = K+1
	}
	return(test[K,4])
}
### Sliced Average Variance Estimator (S.A.V.E.) ####
### obtaining S.A.V.E. kernel matrix by evenly split the response range ###
get_save_matrix_even_range = function(x,y,H){
	z = standardize(x)
	n = length(y)
	p = ncol(x)
	sliced_y = numeric(n)
	slice_prop = numeric(H)
	divpt = c(min(y)-0.1,min(y)+((range(y)[2]-range(y)[1])/H)*c(1:H))
	save_matrix = matrix(0,p,p)
	for(i in 1:H){
		sliced_y[(y<=divpt[i+1])&(y>divpt[i])]=i
		slice_prop[i] = sum(sliced_y[sliced_y==i])/(n*i)
		slice_i = matrix(z[sliced_y==i,],length(z[sliced_y==i,])/p,p)
		if (slice_prop[i]!=0)
		save_matrix = save_matrix + slice_prop[i]*matpower((diag(1,p)- t(slice_i)%*%slice_i/(length(slice_i)/p)),2)

	}
	return(save_matrix)
}
### obtaining S.A.V.E. estimate for a given dimension ###
save_even_range = function(x,y,H,d){
	p = ncol(x)
	M = get_save_matrix_even_range(x,y,H)
	beta_hat=matpower(cov(x),-0.5)%*%(eigen(M)$vectors)[,1:d]
	beta_hat = standardize_col(matrix(beta_hat,p,d))
	return(beta_hat)
}
### estimating dimension of central subspace based on S.A.V.E. kernel matrix ###
library(dr)
save_d = function(x,y,H,alpha){
	a = dr(y~x, method="save",nslice=H)
	b = dr.test(a)
	c = b[b[,4]<0.05,]
	return(length(c[,1]))
}


#######################################################################
#### Section 3: stable procedure for testing structural dimension #####
#######################################################################

sed = function(method,x,y,H,l,w){
	p = ncol(x)
	
   	M = switch(	method,
                 	sir=get_sir_matrix_even_range(standardize(x),y,H), 
                 	phd=get_phd_matrix_r(standardize(x),y),
			save=get_save_matrix_even_range(standardize(x),y,H)
                 )

	svd.result = svd(M)
	eta = svd.result$v
	xi = svd.result$u
	eigen.value = svd.result$d
	lasso.y = vec(M)

	### update eigen values ###
	diff = 1
	step = 0
	while(diff>=1e-5 && step<50){
		step = step + 1
		old.eigen.value = eigen.value
		### ss2: update eta and xi ###
		diff_ss2 = 1
		inner_step = 0
		while(diff_ss2>=1e-5 && inner_step<50){
			inner_step = inner_step + 1
			old.eta = eta
			old.xi = xi
	
			svd.eta = svd(M%*%xi%*%diag(eigen.value))
			eta = svd.eta$u%*%t(svd.eta$v)

			svd.xi = svd(M%*%eta%*%diag(eigen.value))
			xi = svd.xi$u%*%t(svd.xi$v)

			diff_ss2 = max(abs(old.eta - eta)+abs(old.xi-xi))
		}

		### ss1: update eigen.value ###
		lasso.x = matrix(0,(p*p),p)
   		for(i in 1:p){
   			lasso.x[,i] = vec(xi[,i]%*%t(eta[,i]))
 		}
		lasso.x = scale(lasso.x,center=FALSE,scale=1/w)
		ans=cv.glmnet(lasso.x,lasso.y, lambda=c(0,l))
		eigen.value  = (coef(ans, s=l)[-1])*w
		diff = sum(abs(old.eigen.value-eigen.value))
	}

	x.l = matrix(0,(p*p),p)
	for(i in 1:p){
   		x.l[,i] = vec(eta[,i]%*%t(eta[,i]))
 	}
	y.l = vec(M)
	#sigma = diag(w/(abs(replace(eigen.value,eigen.value==0, 1))))
	sigma=diag(w)
	p.l = x.l%*%solve(t(x.l)%*%x.l+p^2*sigma)%*%t(x.l)
	e.l=get_trace(p.l)

	gcv = 1/p^2*(sqrt(t(y.l-x.l%*%eigen.value)%*%(y.l-x.l%*%eigen.value))/(1-e.l/p^2)^2) 
	bic = log(t(y.l-x.l%*%eigen.value)%*%(y.l-x.l%*%eigen.value))+log(p^2)*e.l/p^2

	return(list(round(eigen.value,6),l,min(gcv),min(bic)))
}


tuning.sed = function(method,x,y,H,lambda.range,w){
	crit = matrix(0,length(lambda.range),3)
	for (i in 1:length(lambda.range)){
		result = sed(method,x,y,H,lambda.range[i],w)
		crit[i,1] = result[[2]]
		crit[i,2] = result[[3]]
		crit[i,3] = result[[4]]
	}
	return(crit[crit[,2]==min(crit[,2]),])
}



stable_dimension = function(method,x,y,H,alpha,lambda.range,n_sample,n_weight){
	n = nrow(x)
	p = ncol(x)
   	ini.d = switch(	method,
                 		sir=sir_d_even_range(x,y,H,0.05), 
                 		phd=phd_d_r(x,y,0.05),
				save=save_d(x,y,H,0.05)
                 	)

	opt = tuning.sed(method,x,y,H,lambda.range,rep(1,p))
	result = sed(method,x,y,H,opt[1],rep(1,p))[[1]]
	sed.d = length(result[result!=0])

	sample_d = numeric(n_sample)
	for (j in 1:n_sample){
		selected = sample(1:n,n*0.85,replace=F)
		subx= x[selected,]
		suby = y[selected]

		weight_d = numeric(n_weight)
		for (k in 1:n_weight){
			w = runif(p,alpha,1)
			opt = tuning.sed(method,subx,suby,H,lambda.range,w)
			result =sed(method,subx,suby,H,opt[1],w)[[1]]
			weight_d[k] = length(result[result!=0])
		}
		sample_d[j]=get_mode(weight_d)
	}
	stable.d = get_mode(sample_d)
	return(list(ini.d,sed.d,stable.d))
}



######################################################################
####### Section 4: sparse estimation for basis directions  ###########
######################################################################

### sparse basis direction estimations ####
sparse = function(method,x, y, d, H, w, lambda1,lambda2,delta=0){

   	### parameters ###
	n<-nrow(x)
   	p<-ncol(x)

	G = cov(x)
   	M = switch(	method,
                 	sir=get_sir_matrix_even_range(x,y,H), 
                 	phd=get_phd_matrix(x,y),
			save=get_save_matrix_even_range(x,y,H)
                 ) + delta*G

	m = matpower(M,0.5) 
   	alpha = switch(	method,
                 		sir=sir_even_range(x,y,H,d), 
                 		phd=phd_r(x,y,d),
				save=save_even_range(x,y,H,d)
                 	)
	beta.est = alpha
 	last = beta.est
	iter = 0
	diff = 1
	while (iter<100&diff>1e-3){
		for(i in 1:d) {
     			u = m%*%alpha[,i]
   			x.star=rbind(m, sqrt(lambda2) * matpower(G,-0.5)) 
   			y.star=c(u, rep(0, ncol(x)))
			x.star=scale(x.star,center=FALSE,scale=1/w)

   			# call solvebeta() with lambda2=0
			beta.est[,i] = solvebeta(x.star, y.star, para=c(0, lambda1), sparse="penalty")
			beta.est[,i]=beta.est[,i]*w
  		}

		u = svd(matpower(G,-0.5)%*%M%*%beta.est)$u
		v = svd(matpower(G,-0.5)%*%M%*%beta.est)$v
		alpha = matpower(G,-0.5)%*%u%*%t(v)
		
		iter = iter+1
		diff = max(abs(beta.est - last))
		last = beta.est
	}

	#beta.est=apply(beta.est, 2, norm)
	beta.est = standardize_col(beta.est)

 	# compute objective value
   	comp1<-m %*% solve(G) - m %*% beta.est %*% t(beta.est)
   	rss1<-sum(diag(comp1 %*% G %*% t(comp1)))
   	z<-svd(matpower(G,-0.5) %*% M %*% beta.est)
   	alpha<-matpower(G,-0.5) %*% (z$u) %*% t(z$v)
  	comp2<-m %*% solve(G) - m %*% beta.est %*% t(alpha)
   	rss2<-sum(diag(comp2 %*% G %*% t(comp2)))
   	p.e<-sum(as.vector(beta.est) != 0)

	return(list(beta.est=beta.est,rss1=rss1,rss2=rss2,p.e=p.e))
}

### selecting optimal tuning parameter for sparse basis direction estimations ####
choose_tuning = function(method,x, y, d, H, w, lambda1.range,lambda2.range,delta=0){
	crit = matrix(0,length(lambda1.range)*length(lambda2.range),4)
	rn = 0
	for(j in 1:length(lambda2.range)) {
      	lambda2=lambda2.range[j]
      	for(k in 1:length(lambda1.range)) {
			lambda1=lambda1.range[k] 
			rn = rn + 1

			out = sparse(method,x, y, d, H, w, lambda1,lambda2,delta)

         		aic = n*out$rss1 + 2 * out$p.e
         		bic = n*out$rss1 + log(n) * out$p.e

			crit[rn,1] = lambda1
			crit[rn,2] = lambda2
			crit[rn,3] = aic
			crit[rn,4] = bic
		}
	}
	lambda_aic = crit[crit[,3]==min(crit[,3]),]
	lambda_bic = crit[crit[,4]==min(crit[,4]),]
	return(lambda_aic)
}

### stable sparse basis direction estimations ####
stable_ssdr = function(method,x, y, d, H, w, selection_cut,lambda1.range,lambda2.range,delta,n_sample){

	n = nrow(x)
	p = ncol(x)

   	ini.beta = switch(	method,
                 		sir=sir_even_range(x,y,H,d), 
                 		phd=phd_r(x,y,d),
				save=save_even_range(x,y,H,d)
                 	)

	opt = choose_tuning(method,x, y, d, H, rep(1,p), lambda1.range,lambda2.range,delta)
	sparse.beta = sparse(method,x, y, d, H, rep(1,p), opt[1],opt[2],delta)$beta.est
	sparse.beta = order_beta(ini.beta,sparse.beta)
	sparse.beta = sign_beta(ini.beta,sparse.beta)

	select_sample = beta_sample = matrix(0,p,d)
	for(i in 1:n_sample){
		selected = sample(1:n,n*0.85,replace=F)
		subx= x[selected,]
		suby = y[selected]
		opt = choose_tuning(method,subx, suby, d, H, rep(1,p), lambda1.range,lambda2.range,delta)
		beta.est = sparse(method,subx, suby, d, H, w, opt[1],opt[2],delta)$beta.est
		beta.est = order_beta(ini.beta,beta.est)
		beta.est = sign_beta(ini.beta,beta.est)
		select_sample = select_sample + ceiling(abs(beta.est))
		beta_sample = beta_sample + beta.est

	}
	stable.beta = standardize_col(beta_sample*(select_sample/n_sample>selection_cut))

	return(list(ini.beta,sparse.beta,stable.beta))
}

################################################################################
######  Section 5: nonsparse estimation for basis directions  ##################
################################################################################


### This function contains algorithm to obtain stable nonsparse estimation for basis directions of central subspace ###
stable_nonsparse_estimation = function(method,x,y,H,n_sample,d){
	n = nrow(x)
	p = ncol(x)
	M = matrix(0,p,p)

	ini.beta = switch(	method,
                 		sir=sir_even_range(x,y,H,d), 
                 		phd=phd_r(x,y,d),
				save=save_even_range(x,y,H,d)
                 	)
	step = 0
	for (j in 1:n_sample){
		i = sample(n,0.9*n,replace=F)
		subx= x[i,]
		suby = y[i]

		subd = switch(	method,
                 		sir=sir_d_even_range(subx,suby,H,0.05), 
                 		phd=phd_d_r(subx,suby,0.05),
				save=save_d(subx,suby,H,0.05)
                 	)
		subd

		if (subd == d){
			beta.sub = switch(	method,
                 				sir=sir_even_range(subx,suby,H,d), 
                 				phd=phd_r(subx,suby,d),
						save=save_even_range(subx,suby,H,d)
					)
			M = M + beta.sub%*%t(beta.sub)
			step = step + 1
			#print(j)
		}
	}
	beta.est = matpower(cov(x),-0.5)%*%(eigen(M)$vectors)[,1:d]
	beta.est = standardize_col(matrix(beta.est,p,d))
	return(list(ini.beta,beta.est,step))
}



################################################################################
#####  Section 6: stable procedure for enssemble different number of slices ####
################################################################################

sed_H = function(method,x,y,l,w){
	p = ncol(x)
	
   	M5 = switch(	method,
                 	sir=get_sir_matrix_even_range(standardize(x),y,5), 
                 	phd=get_phd_matrix_r(standardize(x),y),
			save=get_save_matrix_even_range(standardize(x),y,5)
                 )
   	M10 = switch(	method,
                 	sir=get_sir_matrix_even_range(standardize(x),y,10), 
                 	phd=get_phd_matrix_r(standardize(x),y),
			save=get_save_matrix_even_range(standardize(x),y,10)
                 )
   	M15 = switch(	method,
                 	sir=get_sir_matrix_even_range(standardize(x),y,15), 
                 	phd=get_phd_matrix_r(standardize(x),y),
			save=get_save_matrix_even_range(standardize(x),y,15)
                 )
   	M20 = switch(	method,
                 	sir=get_sir_matrix_even_range(standardize(x),y,20), 
                 	phd=get_phd_matrix_r(standardize(x),y),
			save=get_save_matrix_even_range(standardize(x),y,20)
                 )
   	M25 = switch(	method,
                 	sir=get_sir_matrix_even_range(standardize(x),y,25), 
                 	phd=get_phd_matrix_r(standardize(x),y),
			save=get_save_matrix_even_range(standardize(x),y,25)
                 )

	M = (M5 + M10 + M15 + M20 + M25)

	svd.result = svd(M)
	eta = svd.result$v
	xi = svd.result$u
	eigen.value = svd.result$d
	lasso.y = vec(M)

	### update eigen values ###
	diff = 1
	step = 0
	while(diff>=1e-5 && step<50){
		step = step + 1
		old.eigen.value = eigen.value
		### ss2: update eta and xi ###
		diff_ss2 = 1
		inner_step = 0
		while(diff_ss2>=1e-5 && inner_step<50){
			inner_step = inner_step + 1
			old.eta = eta
			old.xi = xi
	
			svd.eta = svd(M%*%xi%*%diag(eigen.value))
			eta = svd.eta$u%*%t(svd.eta$v)

			svd.xi = svd(M%*%eta%*%diag(eigen.value))
			xi = svd.xi$u%*%t(svd.xi$v)

			diff_ss2 = max(abs(old.eta - eta)+abs(old.xi-xi))
		}

		### ss1: update eigen.value ###
		lasso.x = matrix(0,(p*p),p)
   		for(i in 1:p){
   			lasso.x[,i] = vec(xi[,i]%*%t(eta[,i]))
 		}
		lasso.x = scale(lasso.x,center=FALSE,scale=1/w)
		ans=cv.glmnet(lasso.x,lasso.y, lambda=c(0,l))
		eigen.value  = (coef(ans, s=l)[-1])*w
		diff = sum(abs(old.eigen.value-eigen.value))
	}

	x.l = matrix(0,(p*p),p)
	for(i in 1:p){
   		x.l[,i] = vec(eta[,i]%*%t(eta[,i]))
 	}
	y.l = vec(M)
	#sigma = diag(w/(abs(replace(eigen.value,eigen.value==0, 1))))
	sigma=diag(w)
	p.l = x.l%*%solve(t(x.l)%*%x.l+p^2*sigma)%*%t(x.l)
	e.l=get_trace(p.l)

	gcv = 1/p^2*(sqrt(t(y.l-x.l%*%eigen.value)%*%(y.l-x.l%*%eigen.value))/(1-e.l/p^2)^2) 
	bic = log(t(y.l-x.l%*%eigen.value)%*%(y.l-x.l%*%eigen.value))+log(p^2)*e.l/p^2

	return(list(round(eigen.value,6),l,min(gcv),min(bic)))
}


tuning.sed_H = function(method,x,y,lambda.range,w){
	crit = matrix(0,length(lambda.range),3)
	for (i in 1:length(lambda.range)){
		result = sed_H(method,x,y,lambda.range[i],w)
		crit[i,1] = result[[2]]
		crit[i,2] = result[[3]]
		crit[i,3] = result[[4]]
	}
	return(crit[crit[,2]==min(crit[,2]),])
}



stable_dimension_H = function(method,x,y,alpha,lambda.range,n_sample,n_weight){
	n = nrow(x)
	p = ncol(x)
   	ini.d = switch(	method,
                 		sir=sir_d_even_range(x,y,10,0.05), 
                 		phd=phd_d_r(x,y,0.05),
				save=save_d(x,y,10,0.05)
                 	)

	opt = tuning.sed_H(method,x,y,lambda.range,rep(1,p))
	result = sed_H(method,x,y,opt[1],rep(1,p))[[1]]
	sed.d = length(result[result!=0])

	sample_d = numeric(n_sample)
	for (j in 1:n_sample){
		selected = sample(1:n,n*0.85,replace=F)
		subx= x[selected,]
		suby = y[selected]

		weight_d = numeric(n_weight)
		for (k in 1:n_weight){
			w = runif(p,alpha,1)
			opt = tuning.sed_H(method,subx,suby,lambda.range,w)
			result =sed_H(method,subx,suby,opt[1],w)[[1]]
			weight_d[k] = length(result[result!=0])
		}
		sample_d[j]=get_mode(weight_d)
	}
	stable.d = get_mode(sample_d)
	return(list(ini.d,sed.d,stable.d))
}


#####################################################################
################  Section 7:  Examples   ############################
#####################################################################


##### Example 1: estimating structural dimension #####

save_data = function(n,p){
	mu = rep(0,p)
  	sigma = diag(1,p)
  	x <- mvrnorm(n,mu,sigma)
  	epsilon <- rnorm(n, mean = 0, sd = 1)
  	y = rep(0,n)
	for(i in 1:n){
   		y[i] = 2*(x[i,1])^2+(x[i,2])^2+0.25*epsilon[i]
  	}
  	return(list(x,y,epsilon))
}


n = 400
p = 10
d = 2
H = 10
method = "save"
data = save_data(n,p)
x = data[[1]]
y = data[[2]]

lambda.range = seq(0.1,0.3,0.05)
alpha = 0.5
n_sample = 5
n_weight = 2

stable_dimension(method,x,y,H,alpha,lambda.range,n_sample,n_weight)



##### Example 2: sparse estimation of central subsapce #####

lambda1.range = seq(0.001,0.1,0.005)
lambda2.range = 0
n_sample = 5
selection_cut = 0.8
delta = 0.01
stable_ssdr(method,x, y, d, H, rep(1,p), selection_cut,lambda1.range,lambda2.range,delta,n_sample)



##### Example 3: nonsparse estimation of central subsapce #####

data_sir = function(n,p){
	mu = rep(0,p)
  	sigma = diag(1,p)
  	x <- mvrnorm(n,mu,sigma)
  	epsilon <- rnorm(n, mean = 0, sd = 1)
  	y = rep(0,n)
	for(i in 1:n){
    		y[i] = (x[i,1]+x[i,2])^3 + 2*exp(x[i,3]+x[i,4])+ epsilon[i]

  	}
  	return(list(x,y,epsilon))
}

n = 400
p = 10
H = 10
N = 10
n_sample = 300
d = 2

data = data_sir(n,p)
x = data[[1]]
y = data[[2]]

beta = matrix(0,p,2)
beta[1,1] = beta[2,1] = beta[3,2] = beta[4,2] = 1

result = stable_nonsparse_estimation("sir",x,y,H,n_sample,d)
beta_sir = result[[1]]
beta_sir
beta_stable_sir = result[[2]]
beta_stable_sir
result[[3]]
vc(beta_sir,beta)
vc(beta_stable_sir,beta)


##### Example 4: enssemble methods for inverse regression #####

save_data = function(n,p){
	mu = rep(0,p)
  	sigma = diag(1,p)
  	x <- mvrnorm(n,mu,sigma)
  	epsilon <- rnorm(n, mean = 0, sd = 1)
  	y = rep(0,n)
	for(i in 1:n){
   		y[i] = 2*(x[i,1])^2+(x[i,2])^2+0.25*epsilon[i]
  	}
  	return(list(x,y,epsilon))
}


n = 400
p = 10
d = 2

method = "save"
data = save_data(n,p)
x = data[[1]]
y = data[[2]]

lambda.range = seq(0.1,0.3,0.05)
alpha = 0.5
n_sample = 5
n_weight = 2

stable_dimension_H(method,x,y,alpha,lambda.range,n_sample,n_weight)


##### Example 5: Analyzing Hitters' dataset #####



### read in the data ###
data = read.table("Hitter.txt", header=TRUE,na.strings="NA",strip.white=TRUE) 
x = as.matrix(drop(data[,1:16]))
y = as.matrix(drop(data[,18]))

##########################
##### Test Dimension #####
##########################

### set parameter values ###
H = 10
n = length(y)
p = ncol(x)
alpha = 0.5
lambda.range = seq(0.015,0.1,0.01)
n_sample = 5
n_weight = 5
method = "sir"

d.est=stable_dimension(method,x,y,H,alpha,lambda.range,n_sample,n_weight)
d.est[[3]]



####################################
#####   Direction Estimation   #####
####################################

### nonsparse estimation ###
n_sample = 1000
d = 2
nonsparse.est = stable_nonsparse_estimation("sir",x,y,H,n_sample,d)
nonsparse.beta = nonsparse.est[[2]]
nonsparse.beta


### stable estimation ###
lambda1.range = c(seq(0.005,0.1,0.01))
lambda2.range = c(seq(0.005,0.1,0.01))
n_sample = 20
selection_cut = 0.85
delta = 0.01
method = "sir"
d = 2
H = 10

result.stable = stable_ssdr(method,x, y, d, H, rep(1,ncol(x)), selection_cut,lambda1.range,0,delta,n_sample)
ini.beta=result.stable[[1]]
sparse.beta=result.stable[[2]]
stable.beta=result.stable[[3]]

ini.beta
sparse.beta
stable.beta


### compare stability of each method ###
N = 50
vc.ini.beta = vc.nonsparse.beta = vc.sparse.beta = vc.stable.beta =  numeric(N)
for (i in 1:N){
	selected = sample(1:n,n*0.85,replace=F)
	subx= x[selected,]
	suby = y[selected]
	result = stable_ssdr(method,subx, suby, d, H, rep(1,ncol(x)), selection_cut,lambda1.range,0,delta,n_sample)
	vc.ini.beta[i] = vc(result[[1]],ini.beta)
	vc.sparse.beta[i] = vc(result[[2]],sparse.beta)
	
	nonsparse.result = stable_nonsparse_estimation("sir",x,y,H,1000,d)
	vc.nonsparse.beta[i] = vc(nonsparse.result[[2]],nonsparse.beta)
	print(i)
}

for (i in 1:N){
	result = stable_ssdr(method,x, y, d, H, rep(1,ncol(x)), selection_cut,lambda1.range,0,delta,n_sample)
	vc.stable.beta[i] = vc(result[[3]],stable.beta)
	print(i)
}


ini.beta
nonsparse.beta
sparse.beta
stable.beta

1-mean(vc.ini.beta)
1-mean(vc.nonsparse.beta)
1-mean(vc.sparse.beta)
1-mean(vc.stable.beta)












